@@ -4,7 +4,7 @@
 
   Qore Programming Language
 
-  Copyright 2003 - 2016 David Nichols
+  Copyright 2003 - 2017 Qore Technologies, s.r.o.
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -37,10 +37,6 @@
 
 #include <memory>
 #include <typeinfo>
-#include <vector>
-#include <string>
-
-typedef std::vector<std::string> strvec_t;
 
 // postgresql uses an epoch starting at 2000-01-01, which is
 // 10,957 days after the UNIX and Qore epoch of 1970-01-01
@@ -116,18 +112,23 @@ void qore_pg_numeric::toStr(QoreString& str) const {
    for (i = 0; i < ndigits; ++i) {
       if (i == weight + 1)
          str.concat('.');
-      if (i)
+      if (i || weight < 0)
          str.sprintf("%04d", ntohs(digits[i]));
       else
          str.sprintf("%d", ntohs(digits[i]));
       //printd(5, "qore_pg_numeric::toStr() digit %d: %d\n", i, ntohs(digits[i]));
    }
 
-   //printd(5, "qore_pg_numeric::toStr() i: %d weight: %d\n", i, weight);
+   //printd(5, "qore_pg_numeric::toStr() i: %d weight: %d str: %s\n", i, weight, str.c_str());
 
    // now add significant zeros for remaining decimal places
    if (weight >= i)
       str.addch('0', (weight - i + 1) * 4);
+   else if (weight < -1) {
+      for (int i = weight; i < -1; ++i)
+         str.insert("0000", 0);
+      str.prepend(".");
+   }
 
    //printd(5, "qore_pg_numeric::toStr() str: '%s'\n", str.c_str());
 }
@@ -828,20 +829,11 @@ AbstractQoreNode* QorePgsqlStatement::getNode(int row, int col, ExceptionSink *x
    return rv;
 }
 
-QoreHashNode* QorePgsqlStatement::getOutputHash(ExceptionSink* xsink, int* start, int maxrows) {
-   assert(res);
-   ReferenceHolder<QoreHashNode> h(new QoreHashNode, xsink);
-
-   int num_columns = PQnfields(res);
-
-   // assign unique column names
-   strvec_t cvec;
-   cvec.reserve(num_columns);
-
+void QorePgsqlStatement::setupColumns(QoreHashNode& h, strvec_t& cvec, int num_columns) {
    for (int i = 0; i < num_columns; ++i) {
       const char* name = PQfname(res, i);
 
-      HashAssignmentHelper hah(**h, name);
+      HashAssignmentHelper hah(h, name);
       if (*hah) {
          // find a unique column name
          unsigned num = 1;
@@ -859,8 +851,16 @@ QoreHashNode* QorePgsqlStatement::getOutputHash(ExceptionSink* xsink, int* start
       else
          cvec.push_back(name);
 
-      hah.assign(new QoreListNode, xsink);
+      hah.assign(new QoreListNode, 0);
    }
+}
+
+QoreHashNode* QorePgsqlStatement::getOutputHash(ExceptionSink* xsink, bool cols, int* start, int maxrows) {
+   assert(res);
+   ReferenceHolder<QoreHashNode> h(new QoreHashNode, xsink);
+
+   int num_columns = PQnfields(res);
+
 
    //printd(5, "QorePgsqlStatement::getOutputHash() num_columns: %d num_rows: %d\n", num_columns, PQntuples(res));
 
@@ -870,6 +870,14 @@ QoreHashNode* QorePgsqlStatement::getOutputHash(ExceptionSink* xsink, int* start
    int nt = PQntuples(res);
    int max = maxrows < 0 ? nt : (maxrows > nt ? nt : maxrows);
 
+   strvec_t cvec;
+
+   if (cols || (i < max)) {
+      // assign unique column names
+      cvec.reserve(num_columns);
+      setupColumns(**h, cvec, num_columns);
+   }
+
    for (; i < max; ++i) {
       for (int j = 0; j < num_columns; ++j) {
          ReferenceHolder<AbstractQoreNode> n(getNode(i, j, xsink), xsink);
@@ -890,7 +898,7 @@ QoreHashNode* QorePgsqlStatement::getSingleRow(ExceptionSink* xsink, int row) {
    if (!e)
       return 0;
    if (e > 1) {
-      xsink->raiseException("PGSQL-SELECT-ROW-ERROR", "SQL passed to selectRow() returned more than 1 row (%d rows in result set)", e);
+      xsink->raiseException("DBI-SELECT-ROW-ERROR", "SQL passed to selectRow() returned more than 1 row (%d rows in result set)", e);
       return 0;
    }
 
@@ -1167,7 +1175,7 @@ int QorePgsqlStatement::add(const AbstractQoreNode* v, ExceptionSink *xsink) {
                break;
 
             case NT_LIST: {
-               std::auto_ptr<QorePGBindArray> ba(new QorePGBindArray(conn));
+               std::unique_ptr<QorePGBindArray> ba(new QorePGBindArray(conn));
                const QoreListNode* l = reinterpret_cast<const QoreListNode*>(t);
                if (ba->create_data(l, 0, enc, xsink))
                   return -1;
@@ -1223,33 +1231,6 @@ int QorePgsqlStatement::add(const AbstractQoreNode* v, ExceptionSink *xsink) {
       return 0;
    }
 
-   /*
-   if (ntype == NT_LIST) {
-      const QoreListNode* l = reinterpret_cast<const QoreListNode*>(v);
-      int len = l->size();
-      if (!len) {
-         paramTypes[nParams] = 0;
-         paramValues[nParams] = 0;
-      }
-      else {
-         std::auto_ptr<QorePGBindArray> ba(new QorePGBindArray(conn));
-         if (ba->create_data(l, 0, enc, xsink))
-            return -1;
-
-         paramArray[nParams] = 1;
-         paramTypes[nParams] = ba->getArrayOid();
-         paramLengths[nParams] = ba->getSize();
-         pb->ptr = ba->getHeader();
-         paramValues[nParams] = (char *)pb->ptr;
-         paramFormats[nParams] = ba->getFormat();
-         //printd(5, "QorePgsqlStatement::add() array size: %d, arrayoid: %d, data: %p\n", ba->getSize(), ba->getArrayOid(), pb->ptr);
-      }
-
-      ++nParams;
-      return 0;
-   }
-   */
-
    paramTypes[nParams] = 0;
    paramValues[nParams] = 0;
    xsink->raiseException("DBI:PGSQL:EXEC-EXCEPTION", "don't know how to bind type '%s'", v->getTypeName());
@@ -1656,12 +1637,12 @@ int QorePgsqlStatement::parse(QoreString* str, const QoreListNode* args, Excepti
                continue;
             }
             if ((*p) != 'v') {
-               xsink->raiseException("DBI-EXEC-PARSE-EXCEPTION", "invalid value specification (expecting '%v' or '%%d', got %%%c)", *p);
+               xsink->raiseException("DBI-EXEC-PARSE-EXCEPTION", "invalid value specification (expecting '%%v' or '%%d', got %%%c)", *p);
                return -1;
             }
             p++;
             if (isalpha(*p)) {
-               xsink->raiseException("DBI-EXEC-PARSE-EXCEPTION", "invalid value specification (expecting '%v' or '%%d', got %%v%c*)", *p);
+               xsink->raiseException("DBI-EXEC-PARSE-EXCEPTION", "invalid value specification (expecting '%%v' or '%%d', got %%v%c*)", *p);
                return -1;
             }
 
@@ -1773,7 +1754,7 @@ int QorePgsqlStatement::execIntern(const char* sql, ExceptionSink* xsink) {
 
 int QorePgsqlStatement::exec(const QoreString *str, const QoreListNode* args, ExceptionSink *xsink) {
    // convert string to required character encoding or copy
-   std::auto_ptr<QoreString> qstr(str->convertEncoding(enc, xsink));
+   std::unique_ptr<QoreString> qstr(str->convertEncoding(enc, xsink));
    if (!qstr.get())
       return -1;
 
@@ -1860,6 +1841,17 @@ QoreHashNode* QorePGConnection::selectRow(const QoreString *qstr, const QoreList
    return res.getSingleRow(xsink);
 }
 
+AbstractQoreNode* QorePGConnection::select(const QoreString *qstr, const QoreListNode* args, ExceptionSink *xsink) {
+   QorePgsqlStatement res(this, ds->getQoreEncoding());
+   if (res.exec(qstr, args, xsink))
+      return NULL;
+
+   if (res.hasResultData())
+      return res.getOutputHash(xsink, true);
+
+   return new QoreBigIntNode(res.rowsAffected());
+}
+
 AbstractQoreNode* QorePGConnection::exec(const QoreString *qstr, const QoreListNode* args, ExceptionSink *xsink) {
    QorePgsqlStatement res(this, ds->getQoreEncoding());
    if (res.exec(qstr, args, xsink))
@@ -1874,7 +1866,7 @@ AbstractQoreNode* QorePGConnection::exec(const QoreString *qstr, const QoreListN
 AbstractQoreNode* QorePGConnection::execRaw(const QoreString *qstr, ExceptionSink *xsink) {
    QorePgsqlStatement res(this, ds->getQoreEncoding());
    // convert string to required character encoding or copy
-   std::auto_ptr<QoreString> ccstr(qstr->convertEncoding(ds->getQoreEncoding(), xsink));
+   std::unique_ptr<QoreString> ccstr(qstr->convertEncoding(ds->getQoreEncoding(), xsink));
 
    if (res.exec(ccstr->getBuffer(), xsink))
       return NULL;
@@ -1977,7 +1969,7 @@ QoreListNode* QorePgsqlPreparedStatement::fetchRows(int rows, ExceptionSink *xsi
 QoreHashNode* QorePgsqlPreparedStatement::fetchColumns(int rows, ExceptionSink *xsink) {
    if (crow == -1)
       crow = 0;
-   return getOutputHash(xsink, &crow, rows);
+   return getOutputHash(xsink, false, &crow, rows);
 }
 
 QoreHashNode* QorePgsqlPreparedStatement::describe(ExceptionSink *xsink) {
