@@ -23,7 +23,7 @@
 #ifndef _QORE_PGSQL_H
 #define _QORE_PGSQL_H
 
-#include "../config.h"
+#include <config.h>
 
 #include <qore/Qore.h>
 
